#!/usr/bin/env ruby
require 'socket'
require 'pp'

def rgb2ansi(r, g, b)
	if r==g && g==b
		return 16 if r<8
		return 231 if r>248
		return (((r - 8) / 247.0) * 24).round + 232
	end
	
	return 16 + 36*(r/51.0).round + 6*(g/51.0).round + (b/51.0).round
end

IP = ARGV[0]
PORT = 2122
EFFECT = ARGV[1]

puts "Connecting to #{IP}:#{PORT}..."

s = TCPSocket.new(IP, PORT)

puts "Connected."
init = s.recv(3).unpack("C*")

raise "Initial data packet wasn't usable!" if init[2] != 0xFF
puts "Got initial data packet."

dim_x, dim_y = *init
len = dim_x * dim_y * 3 + 3

puts "Size: #{dim_x}x#{dim_y}. Expecting packet length #{len}."

puts "Opening local UDP socket..."
udp = UDPSocket.new
udp.bind("10.10.2.1", 13330)
puts "Waiting for UDP packets on port 13330..."
s.sendmsg("P\x12\x34\x00")
s.sendmsg("E#{EFFECT}\x00") if EFFECT


while 1
	data = udp.recvfrom(1024)[0].unpack("C*")
	#puts "Packet. ID: #{data[0]}, length: #{data.length}"
	raise "Unexpected packet length" unless data.count == len
	raise "Invalid data packet" unless data[len - 1]==0xFF
	id = data.shift << 8 | data.shift
	#next
	#print "."
	puts "\033[2J"
	(0...dim_y).each do |y|
		(0...dim_x).each do |x|
			r, g, b = *data.shift(3)
			color_code = rgb2ansi(r, g, b)
			print "\033[48;5;#{color_code}m "
		end
		puts "\033[0m"
	end
end
